@@ -10,21 +10,27 @@
 #'   available on Windows (as this isn't supported by TensorFlow). Note also
 #'   that since this command runs without privillege the "system" method is
 #'   available only on Windows.
-#' @param version TensorFlow version to install (must be either "latest" or a
-#'   full major.minor.patch specification, e.g. "1.1.0").
-#' @param gpu Install the GPU version of TensorFlow
-#' @param package_url URL of the TensorFlow package to install (if not specified
-#'   this is determined automatically). Note that if this parameter is provied
-#'   then the `version` and `gpu` parameters are ignored.
+#'
+#' @param version TensorFlow version to install. Specify "default" to install
+#'   the CPU version of the latest release. Specify "gpu" to install the GPU
+#'   version of the latest release.
+#'
+#'   You can also provide a full major.minor.patch specification (e.g. "1.1.0"),
+#'   appending "-gpu" if you want the GPU version (e.g. "1.1.0-gpu").
+#'
+#'   Alternatively, you can provide the full URL to an installer binary (e.g.
+#'   for a nightly binary).
+#'
+#' @param extra_packages Additional Python packages to install along with
+#'   TensorFlow.
 #'
 #' @importFrom jsonlite fromJSON
 #'
 #' @export
 install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system"),
-                               version = "latest",
-                               gpu = FALSE,
-                               package_url = NULL,
-                               conda = "auto") {
+                               conda = "auto",
+                               version = "default",
+                               extra_packages = NULL) {
 
   # verify os
   if (!is_windows() && !is_osx() && !is_linux()) {
@@ -49,6 +55,12 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
          call. = FALSE)
   }
 
+  # unroll version
+  ver <- parse_tensorflow_version(version)
+  version <- ver$version
+  gpu <- ver$gpu
+  package_url <- ver$package_url
+
   # flags indicating what methods are available
   method_available <- function(name) method %in% c("auto", name)
   virtualenv_available <- method_available("virtualenv")
@@ -70,7 +82,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
         stop("Conda installation failed (no conda binary found)\n", call. = FALSE)
 
       # do install
-      install_tensorflow_conda(conda, version, gpu, package_url)
+      install_tensorflow_conda(conda, version, gpu, package_url, extra_packages)
 
     } else {
 
@@ -88,7 +100,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
       # if we don't have pip and virtualenv then try for conda if it's allowed
       if ((!have_pip || !have_virtualenv) && have_conda) {
 
-        install_tensorflow_conda(conda, version, gpu, package_url)
+        install_tensorflow_conda(conda, version, gpu, package_url, extra_packages)
 
 
       # otherwise this is either an "auto" installation w/o working conda
@@ -138,7 +150,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
         }
 
         # do the install
-        install_tensorflow_virtualenv(python, virtualenv, version, gpu, package_url)
+        install_tensorflow_virtualenv(python, virtualenv, version, gpu, package_url, extra_packages)
 
       }
     }
@@ -182,7 +194,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
       }
 
       # do the install
-      install_tensorflow_conda(conda, version, gpu, package_url)
+      install_tensorflow_conda(conda, version, gpu, package_url, extra_packages)
 
     } else if (identical(method, "system")) {
 
@@ -197,7 +209,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
       # do system installation
       python <- python_system_version$executable_path
       pip <- file.path(python_system_version$install_path, "Scripts", "pip.exe")
-      install_tensorflow_windows_system(python, pip, version, gpu, package_url)
+      install_tensorflow_windows_system(python, pip, version, gpu, package_url, extra_packages)
 
     } else {
       stop("Invalid/unexpected installation method '", method, "'",
@@ -209,7 +221,7 @@ install_tensorflow <- function(method = c("auto", "virtualenv", "conda", "system
   invisible(NULL)
 }
 
-install_tensorflow_conda <- function(conda, version, gpu, package_url) {
+install_tensorflow_conda <- function(conda, version, gpu, package_url, extra_packages = NULL) {
 
   # create conda environment if we need to
   envname <- "r-tensorflow"
@@ -274,9 +286,12 @@ install_tensorflow_conda <- function(conda, version, gpu, package_url) {
   # install additional packages
   conda_install(envname, tf_extra_pkgs(), conda = conda)
 
+  # install extra packages (use pip to ensure we don't get legacy versions)
+  if (!is.null(extra_packages))
+    conda_install(envname, extra_packages, pip = TRUE, conda = conda)
 }
 
-install_tensorflow_virtualenv <- function(python, virtualenv, version, gpu, package_url) {
+install_tensorflow_virtualenv <- function(python, virtualenv, version, gpu, package_url, extra_packages = NULL) {
 
   # determine python version to use
   is_python3 <- python_version(python) >= "3.0"
@@ -330,12 +345,12 @@ install_tensorflow_virtualenv <- function(python, virtualenv, version, gpu, pack
   pip_install("setuptools", "Upgrading setuptools")
 
   # install tensorflow and related dependencies
-  pkgs <- tf_pkgs(version, gpu, package_url)
+  pkgs <- tf_pkgs(version, gpu, package_url, scipy = TRUE, extra_packages = extra_packages)
   pip_install(pkgs, "Installing TensorFlow")
 }
 
 
-install_tensorflow_windows_system <- function(python, pip, version, gpu, package_url) {
+install_tensorflow_windows_system <- function(python, pip, version, gpu, package_url, extra_packages = NULL) {
 
   # ensure pip is up to date
   cat("Preparing for installation (updating pip if necessary)\n")
@@ -346,7 +361,7 @@ install_tensorflow_windows_system <- function(python, pip, version, gpu, package
   # install tensorflow and dependencies (don't install scipy b/c it requires
   # native code compilation)
   cat("Installing TensorFlow...\n")
-  pkgs <- tf_pkgs(version, gpu, package_url, scipy = FALSE)
+  pkgs <- tf_pkgs(version, gpu, package_url, scipy = FALSE, extra_packages = extra_packages)
   result <- system2(pip, c("install", "--upgrade --ignore-installed",
                            paste(shQuote(pkgs), collapse = " ")))
   if (result != 0L)
@@ -356,6 +371,48 @@ install_tensorflow_windows_system <- function(python, pip, version, gpu, package
 }
 
 
+parse_tensorflow_version <- function(version) {
+
+  # defaults
+  ver <- list(
+    version = "latest",
+    gpu = FALSE,
+    package_url = NULL
+  )
+
+  # full url provided
+  if (identical(version, "default")) {
+
+    # default, no changes required
+
+  # gpu version
+  } else if (identical(version, "gpu")) {
+
+    ver$gpu <- TRUE
+
+  # gpu qualifier provided
+  } else if (grepl("-gpu$", version)) {
+
+    split <- strsplit(version, "-")[[1]]
+    ver$version <- split[[1]]
+    ver$gpu <-TRUE
+
+  # full path to installer binary
+  } else if (grepl("^http", version)) {
+
+    ver$package_url <- version
+
+  # another version
+  } else {
+
+    ver$version <- version
+
+  }
+
+  # return
+  ver
+}
+
 python_unix_binary <- function(bin) {
   locations <- file.path(c("/usr/bin", "/usr/local/bin"), bin)
   locations <- locations[file.exists(locations)]
@@ -387,18 +444,19 @@ python_version <- function(python) {
 
 
 # form list of tf pkgs
-tf_pkgs <- function(version, gpu, package_url, scipy = TRUE) {
+tf_pkgs <- function(version, gpu, package_url, scipy = TRUE, extra_packages = NULL) {
   package <- package_url
   if (is.null(package))
     package <- sprintf("tensorflow%s%s",
                        ifelse(gpu, "-gpu", ""),
                        ifelse(version == "latest", "", paste0("==", version)))
-  c(package, tf_extra_pkgs(scipy = scipy))
+  c(package, tf_extra_pkgs(scipy = scipy, extra_packages = extra_packages))
 }
 
 # additional dependencies to install (required by some features of keras)
-tf_extra_pkgs <- function(scipy = TRUE) {
+tf_extra_pkgs <- function(scipy = TRUE, extra_packages = NULL) {
   pkgs <- c("h5py", "pyyaml",  "requests",  "Pillow")
+  pkgs <- c(pkgs, extra_packages)
   if (scipy)
     c(pkgs, "scipy")
   else
@@ -462,7 +520,7 @@ install_tensorflow_extras <- function(packages, conda = "auto") {
   # an "r-tensorflow" environment (i.e. installed via install_tensorflow())
   if (type %in% c("virtualenv", "conda")) {
     python_binary <- ifelse(is_windows(), "r-tensorflow\\python.exe", "r-tensorflow/bin/python")
-    if (!endsWith(config$python, python_binary)) {
+    if (!grepl(paste0(python_binary, "$"), config$python)) {
       stop("You must be using a version of TensorFlow installed with the ",
            "install_tensorflow() function in order\n",
            "to install packages with the install_tensorflow_extras() function. ",
